function [M,de00] = specCCM(ill,cmf,ssf,reflectance)
    % Computes Compromise Color Matrix from spectral power distribution of
    % illuminant (Mx1) and n reflectances (Mxn) given CMF and SSF curves
    % (Mx3), all sampled at the same M wavelengths. XYZ ill White Point may
    % differ from published values if CMF or range differ from CIE standard.
    % de00 SSF difference from XYZ reference for each patch, top left is 1
    % USAGE: M = specCCM(d55,cmf,d5100,cc24)
    % See: www.strollswithmydog.com/raw-color-space-connection/  v 0.1
    
    ill = ill(:);                       %column-wise data
    cmf = cmf/sum(cmf(:,2).*ill);       %normalize CMF by Y area
    ssf = ssf./sum(ssf.*ill);           %and SSF by respective rgb areas
    
    n = size(reflectance,2);            %number of reflectances
    Ln = reflectance .* ill;            %n radiances from scene
    
    L = Ln(:);                          %make it one long appended array
    SSF = repmat(ssf,n,1);              %for each radiance its own ssf
    CMF = repmat(cmf,n,1);              %and cmf
    
    M = (L .* CMF)' * pinv(L .* SSF)';  %Compromise Color Matrix
    
    %---------------------------------- DE2000 Errors
    rgb = Ln' * ssf;                    %raw data
    xyzRgb = rgb * M';                  %project to XYZ
    xyzRef = Ln' * cmf;                 %reference XYZ tristimulus values
    xyzn = ill' * cmf;                  %reference white point
    
    de00 = deltaE00(xyz2labn(xyzRgb,xyzn),xyz2labn(xyzRef,xyzn));
    
    %---------------------------------- Output
    disp(['XYZ ill White Point: ' num2str(sum(cmf.*ill),4)])
    disp(['Matrix White Point:  ' num2str(sum(M')/sum(M(2,:)),4)]) %#ok<UDIM>
    disp(['Mean dE2000 = ' num2str(mean(de00)) ', Max = '...
        num2str(max(de00)) ' at patch ' num2str(find(de00==max(de00)))])
    disp('Excluding Neutral Patches:')
    disp(['Mean dE2000 = ' num2str(mean(de00(1:18))) ', Max = '...
        num2str(max(de00(1:18))) ' at patch ' num2str(find(de00==max(de00(1:18))))])
    %-----------------------------------
    function lab = xyz2labn(xyz,xyzn)
        %converts Nx3 xyz values to Lab given reference white coords xyzn.
        %by Jack Hogan version 0.12s
        
        fx = f(xyz(:,1)/xyzn(1));
        fy = f(xyz(:,2)/xyzn(2));
        fz = f(xyz(:,3)/xyzn(3));
        
        lab = [116*fy-16, 500*(fx-fy), 200*(fy - fz)];
        
        function y = f(t)
            y = zeros(size(t));
            gt = t > (6/29)^3;
            y(gt) = t(gt).^(1/3);
            y(~gt) =  t(~gt)/3/(6/29)^2 + 4/29;
        end
    end
    %------------------------------------
    function De00=deltaE00(Lab1, Lab2)
        % deltaE00: Calculates the CIE DeltaE 2000 Color Difference
        % adapted to Nx3, matlab 2018.  Original Lawrence Taplin 08Jan01
        % Note: SL,SC&SH (bottom) are based on the second set of Lab values.
        
        %CIELAB Chroma
        C1 = sqrt(Lab1(:,2).^2+Lab1(:,3).^2);
        C2 = sqrt(Lab2(:,2).^2+Lab2(:,3).^2);
        
        %Lab Prime
        mC = (C1+C2)./2;
        G=0.5*(1-sqrt((mC.^7)./((mC.^7)+(25.^7))));
        LabP1 = [Lab1(:,1)  Lab1(:,2).*(1+G)  Lab1(:,3)];
        LabP2 = [Lab2(:,1)  Lab2(:,2).*(1+G)  Lab2(:,3)];
        
        %Chroma
        CP1 = sqrt(LabP1(:,2).^2+LabP1(:,3).^2);
        CP2 = sqrt(LabP2(:,2).^2+LabP2(:,3).^2);
        
        %Hue Angle
        hP1t = atan2d(LabP1(:,3),LabP1(:,2));
        hP1t = hP1t+(hP1t<0).*360;
        hP2t = atan2d(LabP2(:,3),LabP2(:,2));
        hP2t = hP2t+(hP2t<0).*360;
        
        %Add in 360 to the smaller hue angle if absolute value of difference is > 180
        hP1 = hP1t + ((hP1t<hP2t)&(abs(hP1t-hP2t)>180)).*360;
        hP2 = hP2t + ((hP1t>hP2t)&(abs(hP1t-hP2t)>180)).*360;
        
        %Delta Values
        DLP = LabP1(:,1) - LabP2(:,1);
        DCP = CP1 - CP2;
        DhP = hP1 - hP2;
        DHP = 2*(CP1.*CP2).^(1/2).*sind(DhP./2);
        
        %Arithmetic mean of LCh' values
        mLP = (LabP1(:,1)+LabP2(:,1))./2;
        mCP = (CP1+CP2)./2;
        mhP = (hP1+hP2)./2;
        
        %Weighting Functions
        SL = 1+(0.015.*(mLP-50).^2)./sqrt(20+(mLP-50).^2);
        SC = 1+0.045.*mCP;
        T = 1-0.17.*cosd(mhP-30)+0.24.*cosd(2.*mhP)+0.32.*cosd(3.*mhP+6)-0.2.*cosd(4.*mhP-63);
        SH = 1+0.015.*mCP.*T;
        
        %Rotation function
        RC = 2.*sqrt((mCP.^7)./((mCP.^7)+25.^7));
        DTheta = 30.*exp(-((mhP-275)./25).^2);
        RT = -sind(2.*DTheta).*RC;
        
        %Parametric factors
        kL = 1;
        kC = 1;
        kH = 1;
        
        De00 = ((DLP./kL./SL).^2+(DCP./kC./SC).^2+(DHP./kH./SH).^2+(RT.*(DCP./kC./SC).*(DHP./kH./SH))).^(1/2);
    end
end
    
        